#!/usr/bin/python

# This script is meant to be periodically run against all of the X.org
# bugs to modify titles to add details, fix problems, and clarify.
# This is intended to help simplify forwarding the bugs upstream.

from arsenal_lib import *
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = False

def cleanup_distro_name(title):
    # It's unnecessary to have [intrepid], etc. in the title
    re_distro = re.compile("\[(intrepid|jaunty|ubuntu \d+\.\d\d)\] *", re.IGNORECASE)
    if re_distro.search(title):
        return re_distro.sub("", title)
    return title

def cleanup_intel(title, pci_device):
    re_number = re.compile("^\[[89]\d")
    if re_number.search(title):
        return "[i" + title[1:]

    # Fix iGxx
    re_igxx = re.compile("^\[i(gm*[34]\d.*)", re.IGNORECASE)
    m = re_igxx.search(title)
    if m:
        return "[" + m.group(1)

    re_bracket = re.compile("^\[[\w\,]*\d")
    if re_bracket.search(title):
        # Appears to already have been nametagged
        return title

    re_NN5 = re.compile("([89][456]5)")
    m = re_NN5.search(title)
    if m:
        return "[i" + m.group(1) + "] " + title

    re_830 = re.compile("82830")
    if re_830.search(title):
        return "[i830] " + title

    re_g45 = re.compile("(gm*45|4500MHD)", re.IGNORECASE)
    if re_g45.search(title):
        return "[g45] " + title

    re_g35 = re.compile("gm*35", re.IGNORECASE)
    if re_g35.search(title):
        return "[g35] " + title

    # Else, use the pci_device to get the identifier
    if pci_device:
        return "[%s] %s" % (pci_device, title)
    
    return title

def cleanup_ati(title, pci_device):
    if pci_device:
        return "[%s] %s" % (pci_device, title)

    return title

# TODO:  Build a table of ATI and Nvidia hardware names
pci_devices = [
    { 'name':'i810',        're':'(8086:7121)' },
    { 'name':'i810dc',      're':'(8086:7123)' },
    { 'name':'i810e',       're':'(8086:7125)' },
    { 'name':'i815',        're':'(8086:1132|82815)' },
    { 'name':'i830',        're':'(8086:3577|82830)' },
    { 'name':'i845',        're':'(8086:2562|82845G)' },
    { 'name':'i855',        're':'(8086:3582|855GM)' },
    { 'name':'i865',        're':'(8086:2572|82865G)' },
    { 'name':'i915g',       're':'(8086:2582)' },
    { 'name':'i915gm',      're':'(8086:2592|915GM)' },
    { 'name':'i945g',       're':'(8086:2772|945G[ \/]|82945G[ \/])' },
    { 'name':'i945gm',      're':'(8086:27a2|945GM[ \/]|82945GM[ \/])' },
    { 'name':'i945gme',     're':'(8086:27ae|945GME|82945GME)' },
    { 'name':'IGDg',        're':'(8086:a001)' },
    { 'name':'IGDgm',       're':'(8086:a011)' },
    { 'name':'i946gz',      're':'(8086:2972|82946GZ)' },
    { 'name':'g35',         're':'(8086:2982|82G35)' },
    { 'name':'i965q',       're':'(8086:2992|Q965)' },
    { 'name':'i965g',       're':'(8086:29a2|G965)' },
    { 'name':'g33',         're':'(8086:29c2|82G33)' },
    { 'name':'q35',         're':'(8086:29b2)' },
    { 'name':'q33',         're':'(8086:29d2)' },
    { 'name':'i965gm',      're':'(8086:2a02|GM965)' },
    { 'name':'i965gme',     're':'(8086:2a12)' },
    { 'name':'gm45',        're':'(8086:2a42)' },
    { 'name':'IGDeg',       're':'(8086:2e02)' },
    { 'name':'q45',         're':'(8086:2e12)' },
    { 'name':'g45',         're':'(8086:2e22)' },
    { 'name':'g41',         're':'(8086:2e32)' },

    { 'name':'M22',         're':'(1002:5460)' },
    { 'name':'M24-1P',      're':'(1002:3150)' },
    { 'name':'M56GL',       're':'(1002:71c4)' },
    { 'name':'M76',         're':'(1002:9581)' },
    { 'name':'X2300',       're':'(1002:718a)' },
    { 'name':'R420',        're':'(1002:4a54)' },
    { 'name':'R700',        're':'(1002:9443)' },
    { 'name':'R700',        're':'(1002:9441)' },
    { 'name':'RC410',       're':'(1002:5a62)' },
    { 'name':'RS482',       're':'(1002:5975)' },
    { 'name':'RS690',       're':'(1002:791e)' },
    { 'name':'RS690M',      're':'(1002:791f)' },
    { 'name':'RS780M',      're':'(1002:9612)' },
    { 'name':'RV280',       're':'(1002:5964)' },
    { 'name':'RV350',       're':'(1002:4152)' },
    { 'name':'RV370',       're':'(1002:5b63)' },
    { 'name':'RV570',       're':'(1002:7280)' },
    { 'name':'RV610',       're':'(1002:94c1)' },
    { 'name':'RV620',       're':'(1002:95c6)' },
    { 'name':'RV630-AGP',   're':'(1002:9587)' },
    { 'name':'RV630',       're':'(1002:9589)' },
    { 'name':'RV630-XT',    're':'(1002:9588)' },
    { 'name':'RV730XT',     're':'(1002:9490)' },
    { 'name':'HD 3400',     're':'(1002:95c4)' },
    { 'name':'HD 3450',     're':'(1002:95c5)' },
    { 'name':'HD 3600',     're':'(1002:9598)' },
    { 'name':'HD 3650',     're':'(1002:9591)' },
    { 'name':'HD 4850',     're':'(1002:9442)' },
    { 'name':'HD 4870',     're':'(1002:9440)' },
    { 'name':'HD 2400 XT ', 're':'(1002:94c1)' },
    { 'name':'HD 3200',     're':'(1002:9610)' },
    { 'name':'HD 3300',     're':'(1002:9614)' },
    { 'name':'HD 3870',     're':'(1002:9501)' },
    { 'name':'HD 3670',     're':'(1002:9593)' },
    { 'name':'X1400',       're':'(1002:7145)' },
    { 'name':'R300 ND',     're':'(1002:4e44)' },
    { 'name':'R350',        're':'(1002:4e48)' },
    { 'name':'RV410',       're':'(1002:5657)' },
    { 'name':'Xpress 1250', 're':'(1002:7942)' },
    ]
for device in pci_devices:
    device['rc'] = re.compile(device['re'], re.IGNORECASE)

def get_pci_device(desc):
    regex_vga = re.compile('VGA compatible controller (.*)', re.IGNORECASE)

    lines = regex_vga.findall(desc)
    if len(lines) > 0:
        for l in lines:
            if len(l)>0:
                for device in pci_devices:
                    if device['rc'].search(l):
                        return device['name']
                print "Unknown device: '%s'" %(l)
    
    return None
                                            

total_renamed = 0
for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        text = "%-8d  %-12s  %s" % (bug.id, bugtask.status, bug.title)

        pci_device = get_pci_device(bug.description)

        title = cleanup_distro_name(bug.title)

        if pkg == "xserver-xorg-video-intel":
            title = cleanup_intel(title, pci_device)

        if pkg in ("fglrx-installer", "xserver-xorg-video-ati"):
            title = cleanup_ati(title, pci_device)

        if title and title != bug.title:
            if not dryrun:
                bug.bug.title = title
                bug.bug.lp_save()
            print text
            print "  ---------> Renamed to %s" % (title)
            total_renamed += 1

if total_renamed > 0:
    print "Total renamed: ",total_renamed

# TODO:
#   * s/du[ae]l *(head|display)/dual monitor/
#   * s/duel monitor/dual monitor/
#   * s/kayboard/keyboard/
#   * s/start up/startup/
#   * s/ X / Xorg /
#   * s/ res / resolution /
#   * s/ (hang|lock *up) / freeze /
#   * s/ (hangs|locks *up) / freezes /
#   * s/ eating / using /
#   * s/ upon / on /
#   * s/ garbled / corrupted /
#   * s/ sluggish / slow /
#   * s/ sluggishness / slowness /
#   * s/artefact/artifact/
#   * s/doesn\'t/does not/
#   * s/malfunctions/does not work/
#   * s/nonfunctional/does not work/
#   * s/(weird|wrong|craz+y|awful|borked|messed-up|unusable|broken)/incorrect/
#   * s/unreadable/incorrectly displayed/
#   * s/does not startup/fails to start/
#   * s/does not load at startup/fails to start/
#   * s/freezes after logging in/fails to login/
#   * s/screen illumination/backlight/
#   * s/gdm restart/Xorg restart/
#   * s/segmentation fault/SIGSEGV/
#   * s/segfault/SIGSEGV/
#   * s/out of scan range/out of range/
#   Fix common misspellings in title and description
#    * interpid -> intrepid
#    * crach -> crash
