#!/usr/bin/python

# This script is for automatic processing of replies to bugs marked
# Confirmed, to filter out ones where requested info has been provided
# from ones that got incorrectly set to with-response via a
# non-original-reporter's reply.

from arsenal_lib import *
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = True
total_count = 0
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
canonical_kernel_team = lp.launchpad.people['canonical-kernel-team']
bug_limit = 5
total_count = 0
days_expired = 180
message_old_confirmed = """This bug report was marked as Confirmed a while ago but has not had any updated comments for quite some time.  Please let us know if this issue remains in the current Ubuntu release, http://www.ubuntu.com/getubuntu/download .  If the issue remains, click on the current status under the Status column and change the status back to "New".  Thanks.

[This is an automated message.  Apologies if it has reached you inappropriately; please just reply to this message indicating so.]
"""

try:
    for source_pkg in source_pkgs:
        count = 0
        print "Processing ", source_pkg
        src_pkg = d.getSourcePackage(name=source_pkg)
        for bugtask in src_pkg.searchTasks(status="Confirmed"):
            if bug_limit and count >= bug_limit:
                print "Hit max number of bugs to process, exiting."
                break

            bug = ArsenalBug(bugtask.bug, lp.launchpad)
            print "https://bugs.edge.launchpad.net/ubuntu/+bug/%d" % bug.id, "  ", bugtask.status, "  ", bug.title

            if (bug.has_tag("review-request") or
                bug.has_tag("kj-omit") or
                bug.has_tag("ct-rev") or
                bug.has_tag("workflow") or
                bug.has_tag("regression-proposed") or
                bug.has_tag("regression-release") or
                bug.has_tag("regression-update") or
                bug.has_tag("regression-potential")):
                print "Skipping, tagged: ", bug.bug.tags
                continue

            if opt_dryrun:
                print "== Dry Run =="

            # Check for old Confirmed bugs
            days = bug.age_last_message()
            if days > days_expired:
                if not bug.has_tag("kj-triage"):
                   print " --> tagged kj-triage"
                print " --> Msg was: ", message_old_confirmed
                print " --> Incomplete"
                if not opt_dryrun:
                    bug.append_comment(message_old_confirmed)
                    bug.append_tag("kj-triage")
                    bugtask.transitionToStatus(status = "Incomplete")
                count += 1
                total_count += 1
                continue

            # Skip bugs already assigned to canonical-kernel-team or
            # canonical-kernel-team member
            if bugtask.assignee:
                assigned = False
                if (bugtask.assignee.name == canonical_kernel_team.name):
                    print "  --> skipping, bug has an canonical-kernel-team assignee"
                    assigned = True
                else:
                    for person in canonical_kernel_team.getMembersByStatus(status="Approved"):
                        if bugtask.assignee.name == person.name:
                            print "  --> skipping, assigned to", bugtask.assignee.name
                            assigned = True
                            break
                if assigned:
                    continue

            if bug.has_tag("kj-triage"):
                if bug.has_tag("needs-kernel-logs"):
                    print " --> tagged needs-kernel-logs **should not be in a Confirmed state with this tag!**"
                    print " --> tagged review-request"
                    if not opt_dryrun:
                        bug.append_tag("review-request")
                    continue

                if bug.has_tag("needs-upstream-testing"):
                    print " --> tagged needs-upstream-testing, skipping."
                    continue

                # If has tag needs-*, see if those are attached now
                # If so, remove tag.  If no more needs-* tags, set to Triaged
                try:
                    print "--> Triaged"
                    if not opt_dryrun:
                        bugtask.transitionToStatus(status = "Triaged")
                except:
                    print " ERROR:  Failed to update bug ", bug.id
                    print "New status was:  Triaged"
                    raise
                count += 1
                total_count += 1

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
