#!/usr/bin/python

# This script is for automatic processing of replies to bugs marked
# Incomplete, to filter out ones where requested info has been provided
# from ones that got incorrectly set to with-response via a
# non-original-reporter's reply.

from arsenal_lib import *
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = True
total_count = 0
days_expired = 120
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
canonical_kernel_team = lp.launchpad.people['canonical-kernel-team']
bug_limit = 10 # set to 0 or False for no buglimit
total_count = 0
message_close_incomplete = """This bug report was marked as Incomplete and has not had any updated comments for quite some time.  As a result this bug is being closed.  Please reopen if this is still an issue in the current Ubuntu release http://www.ubuntu.com/getubuntu/download . Also, please be sure to provide any requested information that may have been missing.  To reopen the bug, click on the current status under the Status column and change the status back to "New".  Thanks.

[This is an automated message.  Apologies if it has reached you inappropriately; please just reply to this message indicating so.]
"""

message_not_owner_logs = """Thanks for running apport-collect to attach the requested debug files.  We are still waiting for the original bug reporter to attach their debug files so that we can confirm you are indeed seeing the same issue.  If the original bug reporter does not attach their debug files within 7 days we will have you open a separate bug report.  We'll keep you posted.  Thanks for your patience.
"""

message_open_new_bug = """The original bug reporter has not attached their log files.  Please open a new bug report and we'll track your issue there.  Refer to https://wiki.ubuntu.com/KernelTeam/KernelTeamBugPolicies for insturctions on how to properly file a bug against the Ubuntu kernel.  Thanks.
"""

try:
    for source_pkg in source_pkgs:
        count = 0
        print "Processing ", source_pkg

        src_pkg = d.getSourcePackage(name=source_pkg)
        for bugtask in src_pkg.searchTasks(status="Incomplete (with response)", tags=["needs-kernel-logs", "needs-reporter-kernel-logs"]):
            if bug_limit and count >= bug_limit:
                    print "Hit max number of bugs to process, exiting."
                    break

            bug = ArsenalBug(bugtask.bug, lp.launchpad)
            print "https://bugs.launchpad.net/ubuntu/+bug/%d" % bug.id, "  ", bugtask.status, "  ", bug.title

            if (bug.has_tag("review-request") or
                bug.has_tag("kj-omit") or
                bug.has_tag("ct-rev") or
                bug.has_tag("workflow") or
                bug.has_tag("regression-proposed") or
                bug.has_tag("regression-release") or
                bug.has_tag("regression-update") or
                bug.has_tag("regression-potential")):
                print "Skipping, tagged: ", bug.bug.tags
                continue

            # Skip bugs already assigned to canonical-kernel-team or
            # canonical-kernel-team member
            if bugtask.assignee:
                assigned = False
                if (bugtask.assignee.name == canonical_kernel_team.name):
                    print "  --> skipping, bug has an canonical-kernel-team assignee"
                    assigned = True
                else:
                    for person in canonical_kernel_team.getMembersByStatus(status="Approved"):
                        if bugtask.assignee.name == person.name:
                            print "  --> skipping, assigned to", bugtask.assignee.name
                            assigned = True
                            break
                if assigned:
                    continue

            if opt_dryrun:
                print "== Dry Run=="
            # If has tag needs-*, see if those are attached now
            # If so, remove tag.  If no more needs-* tags, set to Confirmed
            needs_kernel_logs = True
            comment = ""
            messages = bugtask.bug.messages_collection

            # Check if apport-collect was ran
            days = 0
            for msg in reversed(messages):
                subject = str(msg.subject)
                if subject.find("apport-collect data") > -1:
                    # Handle the case were apport-collect was run by someone
                    # other than the original bug reporter
                    if (msg.owner_link != bug.bug.owner_link) and not bug.has_tag("needs-reporter-kernel-logs"):
                        greeting = "Hi " + msg.owner.name.split(' ')[0] + ",\n\n"
                        comment = greeting + message_not_owner_logs
                        if opt_dryrun:
                            print " --> tag remove needs-kernel-logs"
                            print " --> tag needs-reporter-kernel-logs"
                            print " --> Msg:", comment
                        else:
                            bug.remove_tag("needs-kernel-logs")
                            bug.append_tag("needs-reporter-kernel-logs")
                            bug.append_comment(comment)
                    elif bug.has_tag("needs-reporter-kernel-logs"):
                        days = bug.age_last_message()
                        continue
                    # Set bug status to Confirmed and remove needs-kernel-logs tag
                    else:
                        if bug.has_tag("needs-kernel-logs"):
                            print " --> remove tag needs-kernel-logs"
                        if bug.has_tag("needs-reporter-kernel-logs"):
                            print " --> remove tag needs-reporter-kernel-logs"
                        print " --> Confirmed"
                        if not opt_dryrun:
                            try:
                                bug.remove_tag("needs-kernel-logs")
                                bug.remove_tag("needs-reporter-kernel-logs")
                                bugtask.transitionToStatus(status = "Confirmed")
                            except:
                                print " ERROR:  Failed to update bug ", bug.id
                                print "New status was:  Confirmed"
                                raise
                        needs_kernel_logs = False
                    count += 1
                    total_count += 1
                    break

            if days > 7:
                print " --> non-reporter apport-collect over 7 days ago"
                if opt_dryrun:
                    print " --> Msg:", message_open_new_bug
                else:
                    bug.append_comment(message_open_new_bug)
                count += 1
                total_count += 1
            # Requires triager to review
#            elif needs_kernel_logs:
#                if opt_dryrun:
#                    print " --> tag review-request"
#                else:
#                   bug.append_tag("review-request")

# Hack to iterate over a specific list of bugs instead.
#       buglist=[]
#       for bug in buglist:
#           bugtask = lp.launchpad.load("https://api.edge.launchpad.net/beta/ubuntu/+source/%s/+bug/%d" % (source_pkg, bug))
#           if bugtask.status != "Incomplete":
#               print "Skipping", bugtask.bug.id
#               continue

        # Expire Incomplete bugs which haven't had an updated comment in 120 days
        for bugtask in src_pkg.searchTasks(status="Incomplete (without response)"):
            print count, bug_limit
            if bug_limit and count >= bug_limit:
                print "Hit max number of bugs to process, exiting."
                break
            bug = ArsenalBug(bugtask.bug, lp.launchpad)
            print "https://bugs.launchpad.net/ubuntu/+bug/%d" % bug.id, "  ", bugtask.status, "  ", bug.title

            if opt_dryrun:
                print "== Dry Run=="

            if len(bug.bug.bug_tasks) > 2:
                print " --> Skipping; bug_tasks ==", len(bug.bug.bug_tasks)
#                if opt_dryrun:
#                    print " --> tagged review-request"
#                else:
#                   bug.append_tag("review-request")
#                count += 1
#                total_count += 1
                continue

            days = bug.age_last_message()

            # Expire bugs which have gone more than 120 days without an
            # updated comment
            if days > days_expired:
                if opt_dryrun:
                    print " --> tagged kj-expired"
                    print " --> Invalid"
                    print " --> Msg was: ", message_close_incomplete
                else:
                    bug.append_comment(message_close_incomplete)
                    bug.append_tag("kj-expired")
                    bugtask.transitionToStatus(status = "Invalid")

                count += 1
                total_count += 1
            else:
                print " -- < 120 days -- skipping"

    print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
