#!/usr/bin/python

# This script is for automatic processing of old Triaged bugs (ie
# last comment more than 180 days ago).

from arsenal_lib import *
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = True
total_count = 0
days_expired = 180
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

total_count = 0
message_old_triaged = """This bug report was marked as Triaged a while ago but has not had any updated comments for quite some time.  Please let us know if this issue remains in the current Ubuntu release, http://www.ubuntu.com/getubuntu/download .  If the issue remains, click on the current status under the Status column and change the status back to "New".  Thanks.

[This is an automated message.  Apologies if it has reached you inappropriately; please just reply to this message indicating so.]
"""
try:
    for source_pkg in source_pkgs:
        count = 0
        print "Processing ", source_pkg

        src_pkg = d.getSourcePackage(name=source_pkg)
        for bugtask in src_pkg.searchTasks(status="Triaged"):
            bug = ArsenalBug(bugtask.bug, lp.launchpad)

            print "https://bugs.edge.launchpad.net/ubuntu/+bug/%d" % bug.id, "  ", bugtask.status, "  ", bug.title

            if (bug.has_tag("review-request") or
                bug.has_tag("kj-omit") or
                bug.has_tag("ct-rev") or
                bug.has_tag("workflow") or
                bug.has_tag("regression-proposed") or
                bug.has_tag("regression-release") or
                bug.has_tag("regression-update") or
                bug.has_tag("regression-potential")):
                print "Skipping, tagged: ", bug.bug.tags
                continue

            if len(bug.bug.bug_tasks) > 3:
                print " --> Skipping; bug_tasks ==", len(bug.bug.bug_tasks)
                continue

            days =  bug.age_last_message()

            # Expire bugs which have gone more than 120 days without an
            # updated comment
            if days > days_expired:
                if opt_dryrun:
                    print "== Dry Run =="
                    if not bug.has_tag("kj-triage"):
                       print " --> tagged kj-triage"
                    print " --> Incomplete"
                    print " --> Msg was: ", message_old_triaged
                else:
                    bug.append_comment(message_old_triaged)
                    if not bug.has_tag("kj-triage"):
                        bug.append_tag("kj-triage")
                    bugtask.transitionToStatus(status = "Incomplete")

                count += 1
                total_count += 1
            else:
                print " -- < 180 days -- skipping"

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
