#!/usr/bin/env python
#
# The purpose of this script it to find and mark duplicates of bug 464552 
#

from arsenal_lib import *
import sys

sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)

#end = int(sys.argv[1])

arsenal = Arsenal()
distro = arsenal.load_project("ubuntu")
opt_dryrun = True

linux = distro.getSourcePackage(name="linux")
#linux = distro.getSourcePackage(name="alsa-driver")
#linux = distro.getSourcePackage(name="pulseaudio")
collection = linux.searchTasks(order_by="-id")[:500]
#collection = linux.searchTasks(order_by="-id", status="New")

count = 0
ttl_count = 0

unknown = 'kernel-series-unknown'
mapping = {
    '10.04':    'lucid',
    '9.10':     'karmic',
    '9.04':     'jaunty',
    '8.10':     'intrepid',
    '8.04':     'hardy'
}
alltags = mapping.values()
#alltags.append(unknown)

kernel_mapping = {
    '2.6.32':   '10.04',
    '2.6.31':   '9.10',
    '2.6.28':   '9.04',
    '2.6.27':   '8.10',
    '2.6.24':   '9.04'
}

re_distro = re.compile("\nDistroRelease: Ubuntu (.*)\n");
re_uname = re.compile("\nUname: Linux (\S+)-\d+-\S+ \S+\n");
re_binary = re.compile("\nBinary package hint: linux\S+-(\d+\.\d+\.\d+)");
for bug_task in collection:
#        if bug_task.bug.id != 486643:
#            continue

	#print count, "/", ttl_count
	ttl_count += 1
	bug = ArsenalBug(bug_task.bug, arsenal.launchpad)

#        print "{0} checking {1}".format(bug.id, end)
#        if bug.id == end:
#            print "DONE"
#            sys.exit(0)

        # If we have a valid version tag then trust it, do not change it.
        tagged = None
        for tag in alltags:
            if bug.has_tag(tag):
                tagged = tag
                break
            if bug.has_tag("kernel-" + tag):
                tagged = tag
                break
        if tagged:
            print "{0} already tagged {1}".format(bug.id, tagged)
            if tagged != unknown and bug.has_tag(unknown):
                print "{0} removing {1}".format(bug.id, unknown)
                bug.remove_tag(unknown)
            if tagged == 'lucid' and bug.has_tag("kernel-lucid"):
                bug.append_tag("lucid")
                bug.remove_tag("kernel-lucid")
            continue

        desc = bug.description.decode("utf-8")
        desc = "\n" + desc + "\n"

        # Scan DistroRelease: for distro version
        distro = None
        m = re_distro.search(desc)
        if m:
            distro = m.group(1)
            print "{0} distro {1}".format(bug.id, distro)

        # Scan Uname: for kernel version
        kernel = None
        m = re_uname.search(desc)
        if m:
            kernel = m.group(1)
            print "{0} has uname {1}".format(bug.id, kernel)

        # Scan Binary package hint: for kernel version
        if not distro and not kernel:
            m = re_binary.search(desc)
            if m:
                kernel = m.group(1)
                print "{0} has binary hint {1}".format(bug.id, kernel)

        # Convert version to a distro if we have no distro
        if not distro and kernel:
            if kernel in kernel_mapping:
                distro = kernel_mapping[kernel]
                print "{0} distro {1} from version {2}".format(bug.id,
                    distro, kernel)

        # Check for apport data
        if not distro and bug.has_tag('apport-collected'):
            data = None
            for m in bug.bug.messages:
                if m.subject == 'apport-collect data':
                    data = m.content
                    break
            if data:
                m = re_distro.search(data)
                if m:
                    distro = m.group(1)
                    print "{0} distro {1} from apport data".format(bug.id, distro)

        tag = unknown
        if distro in mapping:
            tag = mapping[distro]

        if tag and not bug.has_tag(tag):
            print "{0} needs {1}".format(bug.id, tag)
            bug.append_tag(tag)
            if tag != unknown and bug.has_tag(unknown):
                print "{0} removing {1}".format(bug.id, unknown)
                bug.remove_tag(unknown)
        else:
            print "{0} already tagged {1}".format(bug.id, tag)


#        if not distro or not kernel:
#            print "{0} ? - -".format(bug.id)
#            continue
#
#        match = 'B'
#        if distro == '10.04':
#            if kernel == '2.6.32':
#                match = 'G'
#        if distro == '9.10':
#            if kernel == '2.6.31':
#                match = 'G'
#        elif distro == '9.04':
#            if kernel == '2.6.28':
#                match = 'G'
#        elif distro == '8.10':
#            if kernel == '2.6.27':
#                match = 'G'
#        else:
#            match = '?'
#
#        if match == 'B':
#            count += 1
#
#        print "{0} {1} {2} {3}".format(bug.id, match, distro, kernel)

print "{0} / {1}".format(count, ttl_count)
