#!/usr/bin/env python
#
# The purpose of this script it to find and mark duplicates of bug 464552 
#

from arsenal_lib import *

arsenal = Arsenal()
distro = arsenal.load_project("ubuntu")
opt_dryrun = True

automated = "[This is an automated message.  Apologies if it has reached you inappropriately.]"
reply = "The warning reported here typically indicates that it took longer for your system to resume from suspend than expected.  I believe there is a 5 sec barrier which your system likely exceeded."
dmesg_insert = "  Based on your dmesg output I see %s."
dmesg_insert2 = '  Typically you\'ll see something similar to "PM: resume devices took 8.664 seconds" in your dmesg output when this happens.'
reply_end = "  I'm setting this to traiged for further investigation.  Thanks!\n\n"
reply_end2 = "  A patch to prevent this warning has been applied via bug 464552.  As a result I'm marking this as a duplicate to bug 464552.  Please continue to track this issue at that report.  Thanks!\n\n"

def parse_file(file, comment):
	dupe = False
	buffer = file.data.open()
	parse = buffer.read()
	file_text = re.search("(PM: resume devices took (.*) seconds)", parse)
	if file_text:
		print "file text: ",  file_text.group(0)
		time = file_text.group(2).split(".")[0]
		print "timeout: ", time
		comment += dmesg_insert % (file_text.group(0))
		if int(time) < 10:
			comment += reply_end2
			dupe = True
		else:
			comment += reply_end
	else:
		file_text = ""

	return (comment, dupe, file_text)

bug464552 = arsenal.get_bug(464552)
count = 0
dupe_count = 0
triaged_count = 0
ttl_count = 0

linux = distro.getSourcePackage(name="linux")
collection = linux.searchTasks(search_text="suspend_test_finish", status="New")

for bug_task in collection:
	print count, "/", ttl_count
	ttl_count += 1
	oops = ""
	oops_text = ""
	dmesg = ""
	dmesg_text = ""
	current = ""
	current_text = ""
	bug = ArsenalBug(bug_task.bug, arsenal.launchpad)
	comment = "Hi " + bug.owner_firstname.encode('utf-8') + ",\n\n"
	comment += reply
	print "====="
	print bug_task.bug.id, bug_task.bug.title
	if bug.id == 464552:
		print "Skipping 464552"
		continue
#	if bug.has_tag("suspend-test-finish"):
#		print "Skipping, already tagged suspend-test-finish"
#		continue
	if bug.bug.duplicates_collection:
		print "Skipping, bug has duplicates"
		continue
	for attachment in bug_task.bug.attachments:
		if attachment.title == "OopsText.txt":
			oops = attachment
		if attachment.title == "BootDmesg.txt":
			dmesg = attachment
		if attachment.title == "CurrentDmesg.txt":
			current = attachment
	if oops:
		buffer = oops.data.open()
		parse = buffer.read()
		oops_text = re.search("WARNING:.*suspend_test_finish", parse)
		dupe = False
		if oops_text:
			print "oops text: ", oops_text.group(0)
			if dmesg:
				(comment, dupe, dmesg_text) = parse_file(dmesg, comment)
			if current and not dmesg_text:
				(comment, dupe, current_text) = parse_file(current, comment)
			if not current_text and not dmesg_text:
				comment += dmesg_insert2
				comment += reply_end2
				dupe = True
			comment += automated
			if opt_dryrun:
				count +=1
				if dupe:
					print "Dupe 464552"
				else:
					print "Triaged, Medium, tag suspend-test-finish"
				print comment
			else:
				count +=1
				bug.append_comment(comment)
				bug.append_tag("suspend-test-finish")
				if dupe:
					print "Dupe 464552"
					dupe_count += 1
					bug.bug.markAsDuplicate(duplicate_of=bug464552)
				else:
					bug_task.transitionToStatus(status = "Triaged")
					bug_task.transitionToImportance(importance = "Medium")
					triaged_count += 1
					print "Triaged, Medium"
				print comment
		else:
			print "Warning not found in oopstext.txt"
			continue
	else:
		print "OopsText.txt not found"
		continue

print "Dupes: ", dupe_count
print "Triaged: ", triaged_count
