#!/usr/bin/python

# This script is for automatic processing of replies to bugs marked
# Incomplete, to filter out ones where requested info has been provided
# from ones that got incorrectly set to with-response via a
# non-original-reporter's reply.

from arsenal_lib import *
from lpltoolkit import *

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = 0
total_count = 0
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

total_count = 0

regex_xorglog          = re.compile('xorg.*log',           re.IGNORECASE)
regex_conf             = re.compile('conf',                re.IGNORECASE)
regex_backtrace        = re.compile('ktrace',              re.IGNORECASE)
regex_lspci            = re.compile('lspci',               re.IGNORECASE)


try:
    for source_pkg in source_pkgs:
        count=0
        print "Processing ", source_pkg
        for bugtask in d.searchTasks(search_text=source_pkg, status="Incomplete (with response)"):
            if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
                continue

            bug = ArsenalBug(bugtask.bug, lp.launchpad)

            if len(bug.attachments) < 1:
                continue

            print bug.id, "  ", bugtask.status, "  ", bug.title

            # TODO:  Some of this attachment handling logic should be done in arsenal_lib
            #        Probably needs a new ArsenalAttachment wrapper class
            # TODO:  It would be nice to define pre-conditions in a 'rules' file, that
            #        arsenal could then apply to do triaging more programmatically
            has_xorglog = False
            has_lspci = False
            for a in bug.attachments:
                author = a.message.owner.name
                if author != bug.bug.owner.name:
                    continue

                try:
                    hosted_file = a.data
                    hosted_file_buffer = hosted_file.open()
                    t = hosted_file_buffer.filename

                    if regex_xorglog.search(t):
                        has_xorglog = True
                    if regex_lspci.search(t):
                        has_lspci = True
                except:
                    print "*** Exception ***  Failed to get contents of attachment"

                # Is a patch attached?  If so, maybe set to Confirmed directly
            
            # If has tag needs-*, see if those are attached now
            # If so, remove tag.  If no more needs-* tags, set to Confirmed
            got_xorglog = False
            got_lspci_vvnn = False
            if bug.has_tag('needs-xorglog') and has_xorglog == 1:
                bug.remove_tag('needs-xorglog')
                print " --> remove needs-xorglog"
                got_xorglog = True

            if bug.has_tag('needs-lspci-vvnn') and has_lspci == 1:
                bug.remove_tag('needs-lspci-vvnn')
                print " --> removed needs-lspci-vvnn"
                got_lspci_vvnn = True

            # If all conditions met, set to Confirmed
            if (got_xorglog or got_lspci_vvnn) and \
               (has_xorglog and has_lspci):
                new_status = "Confirmed"
                try:
                    bugtask.transitionToStatus(status = new_status)
                    print " ---> " + new_status
                    count += 1
                    total_count += 1
                except:
                    print " ERROR:  Failed to update bug ", bug.id
                    print "New status was:  ", new_status
                    raise

            # TODO:  Review comments since bug was set to Incomplete
            #        Note the original reporter
            #        If no one replying to the bug was the original reporter
            #        If all non-original-reporter repliers are bug triagers, reset to Incomplete
            #        Has the original reporter said "fixed"/"works now"/"can be closed"?
            #        If has lots of commenters, add caution that many people may simply
            #          be seeing the same symptoms, not the same bug.  Tag it.

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
