#!/usr/bin/python

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
total_count = 0
tagged      = 0
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = False

tags_regexes = [
    {'name':'kubuntu',   're':'(kubuntu|kdm|kwin|\bkde|konqueror|konsole)', },
    {'name':'ppc',       're':'(powerpc|[\[\b]ppc[\[\b])', },
    {'name':'karmic',    're':'(karmic|[^0-9\.\-]9\.10[^0-9\.\-])', },
    {'name':'jaunty',    're':'(jaunty|[^0-9\.\-]9\.04[^0-9\.\-])', },
    {'name':'intrepid',  're':'(intrepid|[^0-9\.\-]8\.10[^0-9\.\-])', },
    {'name':'hardy',     're':'(hardy|[^0-9\.\-]8\.04[^0-9\.\-])', },
]
for tag_regex in tags_regexes:
    tag_regex['rc'] = re.compile(tag_regex['re'], re.IGNORECASE)

# TODO:  Enable this to apply to more than just -intel
for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        if bug.has_tag("omit"):
            continue

        total_count += 1
        was_tagged = False
        title = bug.title.decode("utf-8")
        desc = bug.description.decode("utf-8")
        print "%-7s %-11s %s" % (bug.id, bugtask.status, title.encode("utf-8"))

        if bugtask.importance == "Wishlist":
            print "  --> Wishlist"
            continue

        # If we found the release in the title, mark it
        for tag in tags_regexes:
            if tag['rc'].search(title):
                was_tagged = True
                print "  --> title: ",tag['name']
                if tag['name'] in bug.bug.tags:
                    print "  ----> Already tagged ",tag['name']
                    continue
                if not dryrun:
                    bug.append_tag(tag['name'])
                tagged += 1
                # Take as many matches as are present

        if was_tagged:
            continue

        # If still no match, also try looking in description
        for tag in tags_regexes:
            if tag['rc'].search(desc):
                print "  --> desc: ",tag['name']
                if not dryrun:
                    bug.append_tag(tag['name'])
                tagged += 1
                break  # Only take the first match

print total_count," total bugs, ",tagged," tagged"

