#!/usr/bin/python

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
total_count = 0
tagged      = 0
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = True

tags_regexes = [
    {'name':'error-exit',    're':'(exits with|server terminated|kills session)', },
    {'name':'error-exit',    're':'(forces a logout)', },
    {'name':'no-screens',    're':'(no screens found|screen not detected)', },
    {'name':'crash',         're':'(crash|gdm_slave_xioerror_handler)', },
    {'name':'freeze',        're':'(freeze|freezing| hangs* |locks* *up|system unresponsive)', },
    {'name':'corruption',    're':'(corrupt|rubbish|garbage|garbled|display bugs|colored noise)', },
    {'name':'tearing',       're':'(tearing)', },
    {'name':'resolution',    're':'(resolution| mode )', },
    {'name':'performance',   're':'(performance|sluggish|jerky|slow at redraw)', },
    {'name':'flicker',       're':'(flicker|blinking|blinks|flashes periodically)', },
    {'name':'dual-head',     're':'(dual|ext|external)[ -]*(head|monitor)', },
    {'name':'backlight',     're':'(backlight|brightness)', },
    {'name':'videoplayback', 're':'( Xv |mplayer|xine|totem)', },
    {'name':'ghost-monitor', 're':'(false|non-existant) (monitor|vga )', },
    {'name':'3d',            're':'(3D |OpenGL| GL |Blender)', },
    {'name':'compiz',        're':'( compiz|desktop effects)', },
    {'name':'resume',        're':'(suspend|resume|hibernate|wake up)', },
    {'name':'cropped',       're':'(cropped)', },
    {'name':'font-size',     're':'(text .* too large|huge font)', },
    {'name':'black-screen',  're':'(black|blank) (tv |video|screen)', },
    {'name':'black-screen',  're':'screen (goes|stays) (black|blank)', },
    {'name':'vt-switch',     're':'(VT switch|ctrl-alt-f|tty switch|switch .*to console)', },
    {'name':'tv-out',        're':'(TV[ -]*Out|S[ -]*Video| HDMI )', },
    {'name':'high-cpu',      're':'(high \%*cpu|cpu \d+\%|keeps processor.*busy|high load)', },
]
for tag_regex in tags_regexes:
    tag_regex['rc'] = re.compile(tag_regex['re'], re.IGNORECASE)

# TODO:  Enable this to apply to more than just -intel
for pkg in source_pkgs:
    s = d.getSourcePackage(name = pkg)
    for bugtask in s.searchTasks():
        bug = ArsenalBug(bugtask.bug, lp.launchpad)
        if bug.has_tag("omit"):
            continue

        total_count += 1
        title = bug.title.decode("utf-8")
        desc = bug.description.decode("utf-8")
        print "%-7s %-11s %s" % (bug.id, bugtask.status, title)

        # If we found the driver listed in Xorg.0.log, transition to it
        for tag in tags_regexes:
            if tag['name'] in bug.bug.tags:
                print "  --> Already tagged ",tag['name']
                continue

            if tag['rc'].search(title):
                if not dryrun:
                    bug.append_tag(tag['name'])
                tagged += 1

print total_count," total bugs, ",tagged," tagged"

