#!/usr/bin/python

# The purpose of this script is to move bugs out of the
# xorg package into more appropriate ones

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError
from lpltoolkit import LaunchpadService
from cStringIO import StringIO
from gzip import GzipFile

total_count = 0
repackaged  = 0
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")
dryrun      = False

drivers = {
    'openchrome': 'xserver-xorg-video-openchrome',
    'intel':      'xserver-xorg-video-intel',
    'nv':         'xserver-xorg-video-nv',
    'nouveau':    'xserver-xorg-video-nouveau',
    'nvidia':     'nvidia-graphics-drivers-180',
    'fglrx':      'fglrx-installer',
    'ati':        'xserver-xorg-video-ati',
    'radeon':     'xserver-xorg-video-ati',
    'radeonhd':   'xserver-xorg-video-radeonhd',
    'savage':     'xserver-xorg-video-savage',
    'sis':        'xserver-xorg-video-sis',
    'r128':       'xserver-xorg-video-r128',
    'mga':        'xserver-xorg-video-mga',
    'trident':    'xserver-xorg-video-trident',
    'psb':        'xserver-xorg-video-psb',
    'vboxvideo':  'virtualbox-ose',
    }

# TODO:  layout -> xkeyboard-config
# TODO:  Catch Via/Openchrome bugs
title_regexes = [
    {'name':'synaptics',    're':'(mouse|touch) *pad',      'src':'xserver-xorg-input-synaptics'},
    {'name':'evdev(kbd)',   're':' (key|keys|keyboard)',    'src':'xserver-xorg-input-evdev'},
    {'name':'evdev(kbd)',   're':'(key|keys|keyboard) ',    'src':'xserver-xorg-input-evdev'},
    {'name':'evdev(mouse)', 're':'mouse.*(input|events|click|button|pointer)', 'src':'xserver-xorg-input-evdev'},
    {'name':'Openchrome',   're':'openchrome',              'src':'xserver-xorg-video-openchrome'},
    {'name':'Intel',        're':'intel',                   'src':'xserver-xorg-video-intel'},
    {'name':'i9xx',         're':'(965gm|945)',             'src':'xserver-xorg-video-intel'},
    {'name':'Nvidia',       're':'(nvidia|GeForce)',        'src':'nvidia-graphics-drivers-180'},
    {'name':'Radeonhd',     're':'radeonhd',                'src':'xserver-xorg-video-radeonhd'},
    {'name':'Fglrx',        're':'fglrx',                   'src':'fglrx-installer'},
    {'name':'Ati',          're':' (radeon|ati)',           'src':'xserver-xorg-video-ati'},
    {'name':'Ati',          're':'(radeon|ati) ',           'src':'xserver-xorg-video-ati'},
    {'name':'Via',          're':' via ',                   'src':'xserver-xorg-video-openchrome'},
    {'name':'mesa',         're':'mesa ',                   'src':'mesa'},
    {'name':'mesa(gl)',     're':'(opengl| gl )',           'src':'mesa'},
    {'name':'mesa(de)',     're':'(compiz| kwin)',          'src':'mesa'},
    {'name':'mesa(de)',     're':'desktop effects',         'src':'mesa'},
    {'name':'xserver',      're':'(xserver|x server)',      'src':'xorg-server'},
    {'name':'xserver(seg)', 're':'(crash|seg.*fault)',      'src':'xorg-server'},
    {'name':'xserver(abt)', 're':'(abort|restart)',         'src':'xorg-server'},
    {'name':'xserver(gdm)', 're':'gdm_slave_xioerror',      'src':'xorg-server'},
    {'name':'xserver(rez)', 're':'resolution',              'src':'xorg-server'},
    {'name':'xephyr',       're':'xephyr',                  'src':'xorg-server'},
    {'name':'wacom',        're':'wacom',                   'src':'wacom-tools'},
]
for title_regex in title_regexes:
    title_regex['rc'] = re.compile(title_regex['re'], re.IGNORECASE)

pci_devices = [
    { 'name':'ati',         're':'ATI Technologies '},
    { 'name':'nvidia',      're':'(nvidia|GeForce)'},
    { 'name':'intel',       're':'Intel Corporation'},
    { 'name':'openchrome',  're':'VIA Technologies'},
    ]
for device in pci_devices:
    device['rc'] = re.compile(device['re'], re.IGNORECASE)

regex_highcpu     = re.compile('(\d\d%.*cpu|high cpu)', re.IGNORECASE)
regex_proprietary = re.compile('(proprietary|propriety|proptietary|closed source|closed driver|closed ati)', re.IGNORECASE)

regex_nonfree_kernel_fglrx  = re.compile('(NonfreeKernelModules|SourcePackage): *fglrx')
regex_nonfree_kernel_nvidia = re.compile('(NonfreeKernelModules|SourcePackage): *nvidia')
regex_checkbox_resolution   = re.compile('CheckboxTest: resolution')
regex_checkbox              = re.compile('CheckboxTest: .*')
regex_failsafe              = re.compile('failsafe', re.IGNORECASE)
regex_workflow              = re.compile('(Please|depend|apport)', re.IGNORECASE)

def transitionToTarget(bugtask, package_name):
    if package_name != None:
        package = d.getSourcePackage(name=package_name)
        if package == None:
            print "Error:  Could not find source package ",package_name
            return False
    
        if not dryrun:
            try:
                bugtask.transitionToTarget(target = package)
            except HTTPError, e:
                # TODO:  Need to handle this better
                print "exception encountered: %s" %(e)
                exit

    return True

def get_pci_device(desc):
    regex_vga = re.compile('VGA compatible controller: (.*)$', re.IGNORECASE)

    lines = regex_vga.findall(desc)
    if len(lines) > 0:
        for l in lines:
            for device in pci_devices:
                if device['rc'].search(l):
                    return device['name']

    return None

def get_driver_from_xorglog(bug):
    regex_xorglog = re.compile("xorg.*log", re.IGNORECASE)
    regex_gz = re.compile("\.gz$", re.IGNORECASE)
    regex_driver = re.compile("\(II\) Loading /usr/lib/xorg/modules/drivers/+(.*)_drv\.so")
    drivers = []
    for a in bug.attachments:
        if not regex_xorglog.search(a.title):
            continue

        # if author != bug original reporter, skip it
        if a.message.owner.name != bug.owner.name:
            continue
        
        hosted_file = a.data
        hosted_file_buffer = hosted_file.open()
        content = hosted_file_buffer.read()

        # Decompress if necessary
        if regex_gz.search(a.title):
            content = GzipFile('', 'r', 0, StringIO(content)).read()

        # Parse data, look for driver
        found_drivers = regex_driver.findall(content)

        video_driver = None
        for d in found_drivers:
            if video_driver and video_driver != d:
                # Ignore the radeon driver, it can co-exist with ati
                if video_driver == 'radeon' and d == 'ati':
                    continue
                elif video_driver == 'ati' and d == 'radeon':
                    continue

                # Prefer non-vesa driver if available
                if d == 'vesa' or d == 'fbdev':
                    continue
                elif d and (video_driver == 'vesa' or video_driver == 'fbdev'):
                    video_driver = d
                    continue
                
                # Ambiguous; multiple video drivers are loaded.  Needs human review
                print "multiple video drivers are loaded (%s and %s)"%(video_driver, d)
                return None
            video_driver = d
        
        if video_driver:
            drivers.append(video_driver)

    last_driver = None
    for d in drivers:
        if last_driver != None and d != last_driver:
            # Prefer non-vesa driver if available
            if d == 'vesa' or d == 'fbdev':
                continue
            elif d and (last_driver == 'vesa' or last_driver == 'fbdev'):
                last_driver = d
                continue

            # Ambiguous; some logs show one driver, others another.  Needs human review
            print "Logs show differing drivers (%s and %s)" %(last_driver, d)
            return None
        last_driver = d

    return last_driver

def get_driver_from_xorgconf(bug):
    regex_xorgconf = re.compile("xorg.*conf", re.IGNORECASE)
    regex_driver = re.compile("Driver\s+\"(\w+)\"")
    drivers = []
    for a in bug.attachments:
        if not regex_xorgconf.search(a.title):
            continue
        
        # if author != bug original reporter, skip it
        if a.message.owner.name != bug.owner.name:
            continue
        
        hosted_file = a.data
        hosted_file_buffer = hosted_file.open()
        content = hosted_file_buffer.read()

        # Parse data, look for driver
        found_drivers = regex_driver.findall(content)

        video_driver = None
        for d in found_drivers:
            if video_driver and video_driver != d:
                # Ambiguous; multiple video drivers are loaded.  Needs human review
                return None
            video_driver = d
        
        if video_driver:
            drivers.append(video_driver)

    last_driver = None
    for d in drivers:
        if last_driver != None and d != last_driver:
            # Ambiguous; some logs show one driver, others another.  Needs human review
            return None
        last_driver = d
    
    return last_driver

s = d.getSourcePackage(name = "xorg")
for bugtask in s.searchTasks(assignee=None, importance="Undecided"):
    if bugtask.bug_target_display_name != "xorg (Ubuntu)":
        continue

    bug = ArsenalBug(bugtask.bug, lp.launchpad)

    # Skip bugs that cause 500 errors
    if bug.id == 377090:
        continue

    title = bug.title.decode("utf-8")
    desc = bug.description.decode("utf-8")
    pci_device = get_pci_device(desc)
    driver = get_driver_from_xorglog(bugtask.bug)
    driver_conf = get_driver_from_xorgconf(bugtask.bug)
    text = "%-7s %-11s %-8s %s" % (bug.id, bugtask.status, driver, title)

    # Bugs with a significant status should stay
    if bugtask.status == "Triaged" or bugtask.status == "In Progress" or bugtask.status == "Fix Committed":
        continue

    # Bugs that look like they really belong to xorg should stay
    if regex_failsafe.search(title) or regex_workflow.search(title):
        continue

    total_count += 1

    # Undouble bugs reported against another component
    is_doubled = False
    for t in bug.bug_tasks:
        if t.bug_target_display_name == "xorg (Ubuntu)":
            continue
        if t.is_complete:
            continue
        if t.bug_target_display_name.find("(Ubuntu)") == -1:
            continue
        # TODO: Check if the task is against an xorg package
        #is_doubled = True

    if is_doubled:
        if not dryrun:
            message_undouble = "[No need to have this double-filed against xorg as well]"
            bug.append_comment(message_undouble)
            bugtask.transitionToStatus(status = "Invalid")
        print "Doubled!     -->  ", text.encode("utf-8")
        repackaged += 1
        continue

    # Checkbox bugs
    if regex_checkbox.search(desc):
        print "%-12s -->  " % ("checkbox"), text.encode("utf-8")
#        transitionToTarget(bugtask, "nvidia-graphics-drivers-180")
#        repackaged += 1
        continue

    # If we found the driver listed in Xorg.0.log, transition to it
    if driver in drivers:
        print "%-12s -->  " % (driver), text.encode("utf-8")
        transitionToTarget(bugtask, drivers[driver])
        repackaged += 1
        continue

    # If we found the driver listed in xorg.conf, transition to it
    if driver_conf in drivers:
        print "%-12s -+>  " % (driver_conf), text.encode("utf-8")
        transitionToTarget(bugtask, drivers[driver_conf])
        repackaged += 1
        continue

    title_match = False
    for title_regex in title_regexes:
        if title_regex['rc'].search(title):
            print "%-12s -->  " % (title_regex['name']), text.encode("utf-8")
            transitionToTarget(bugtask, title_regex['src'])
            repackaged += 1
            title_match = True
            break
    if title_match:
        continue

    if pci_device == 'intel':
        print "%-12s ==>  " % (pci_device), text.encode("utf-8")
        transitionToTarget(bugtask, "xserver-xorg-video-intel")
        repackaged += 1
        continue

    if regex_nonfree_kernel_nvidia.search(desc):
        print "%-12s -->  " % ("nvidia"), text.encode("utf-8")
        transitionToTarget(bugtask, "nvidia-graphics-drivers-180")
        repackaged += 1
        continue

    if regex_nonfree_kernel_fglrx.search(desc):
        print "%-12s -->  " % ("fglrx"), text.encode("utf-8")
        transitionToTarget(bugtask, "fglrx-installer")
        repackaged += 1
        continue

    if regex_proprietary.search(desc) and \
           (pci_device == 'nvidia' or pci_device == 'fglrx'):
        print "%-12s ==>  " % (pci_device), text.encode("utf-8")
        transitionToTarget(bugtask, drivers[pci_device])
        repackaged += 1
        continue

    print " ???              ", text.encode("utf-8")

if repackaged > 0:
    print total_count," total bugs, ",repackaged," re-packaged"

