#!/usr/bin/python

# The purpose of this script is to report status of upstreamed bugs,
# principly for use in working with Intel on issues that are important
# to Ubuntu.
#
# A secondary purpose of this script (which may be worth breaking out
# separately) is to identify work-tasks relating to upstream[able] bugs.

from arsenal_lib import *
from lpltoolkit import LaunchpadService
import string

source_pkg  = "xserver-xorg-video-intel"
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

re_kubuntu = re.compile("Kubuntu:", re.IGNORECASE)
re_needs = re.compile("\(Needs")

new = 0
incomplete = 0
confirmed = 0
triaged = 0
upstreamed = 0
needs_upstreamed = 0
other = 0
fixed_upstream = 0
open_upstream = 0

print "---- Ubuntu ----   ---------- FDO ----------"
print "%-7s %-9s  %-6s %-9s %-9s  %s" % ("LP#", "Priority", "FDO#", "Status", "Priority", "Description")

s = d.getSourcePackage(name = source_pkg)
for bugtask in s.searchTasks():
    bug = ArsenalBug(bugtask.bug, lp.launchpad)
    title = bug.title

    if bug.has_tag('omit') or bug.has_tag('kubuntu'):
        continue
    
    upstream_bug = ''
    for watch in bugtask.bug.bug_watches:
        if watch.bug_tracker.name != "freedesktop-bugs":
            continue
        upstream_bug = watch.remote_bug
        if watch.remote_status == "RESOLVED FIXED" or watch.remote_status == "RESOLVED DUPLICATE":
            fixed_upstream += 1
        elif watch.remote_status != None:
            open_upstream += 1

            upstream_status = string.split(watch.remote_status, ' ').pop()

            upstream_importance = watch.remote_importance
            if upstream_importance == 'UNKNOWN':
                upstream_importance = '?'

            # LP: 389269 - launchpad is not providing remote_importance on bugs
            print "%-7s %-9s  %-6s %-9s %-9s  %s" % (bug.id, bugtask.importance, upstream_bug,
                                                       upstream_status, upstream_importance, title)
            continue

    if bugtask.status == "New":
        new += 1
    elif bugtask.status == "Incomplete":
        incomplete += 1
    elif bugtask.status == "Confirmed":
        confirmed += 1
    elif bugtask.status == "Triaged":
        triaged += 1
        if not upstream_bug:
            needs_upstreamed += 1
    else:
        other += 1

print
print "%2d bugs are unprocessed" % (new)
print "%2d bugs are waiting on reporters for more information" % (incomplete)
print "%2d bugs are pending Ubuntu review" % (confirmed)
print "%2d bugs need upstreamed" % (needs_upstreamed)
print "%2d bugs are in process by Ubuntu" % (other)
print
print "%2d bugs are fixed upstream" % (fixed_upstream)
print "%2d bugs are waiting on upstream" % (open_upstream)

