#!/usr/bin/python

import simplejson as json
import socket
from arsenal_lib import *
from lpltoolkit import LaunchpadService

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

total_count = 0
source_pkgs = sys.argv[1:]
lp          = LaunchpadService()
d           = lp.load_project("ubuntu")

# TODO:  It would be interesting to parse these off https://wiki.ubuntu.com/X/Tagging
all_symptoms = [
    'error-exit',
    'no-screens',
    'crash',
    'freeze',

    'corruption',
    'tearing',
    'resolution',
    'performance',

    'dual-head',
    'backlight',
    'videoplayback',
    'ghost-monitor',
    'edid',
    '3d',
    'compiz',
    'resume',
    'cropped',

    'font-size',
    'black-screen',
    'vt-switch',
    'tv-out',
    'high-cpu',
    ]

regex_chipset = re.compile('^\[(\w+).*?\] *(.*)$', re.IGNORECASE)

bugs = []
try:
    for source_pkg in source_pkgs:
        s = d.getSourcePackage(name = source_pkg)
        for bugtask in s.searchTasks():

            bug = ArsenalBug(bugtask.bug, lp.launchpad)
            bug.source_pkg = source_pkg
            symptoms = []
            
            if bug.has_tag('omit') or bug.has_tag('kubuntu'):
                continue

            for tag in bugtask.bug.tags:
                if tag in all_symptoms:
                    symptoms.append(tag)

            m = regex_chipset.match(bug.title)
            if m:
                chip = m.group(1)
                title = m.group(2)
            else:
                chip = ''
                title = bug.title
                
            bug = {
                'id'         : bug.id,
                'title'      : title,
                'package'    : source_pkg,
                'symptoms'   : symptoms,
                'chip'       : chip,
                'tags'       : bugtask.bug.tags,
                'status'     : bugtask.status,
                'importance' : bugtask.importance
                }
            bugs.append(bug)

    print json.dumps(bugs, indent=4)

except socket.error as e:
    excType, excValue, excTraceback = sys.exc_info()
    sys.stderr.write("  %s\n" %(excValue.response))
    sys.stderr.write("  %s\n" %(excValue.content))
    sys.stderr.write("Launchpad service connection timed out")
    sys.exit(1)

except HTTPError as e:
    sys.stderr.write("HTTPError exception hit\n")
    #print e.contents
    sys.stderr.write("%s\n" %(e.response))
    sys.exit(1)

except:
    sys.stderr.write("Unknown exception hit\n")
    raise
